<?php

/**
 * Make init
 *
 * @package wp-cli
 */
class Init_Command extends WP_CLI_Command {
    /**
     * Secure instance by default set of operations
     * [--format=<format>]
     * : The format to use when printing the user; acceptable values:
     * [--checkers=<checkers>]
     * : List of checkers.
     * ## EXAMPLES
     *
     *      wp init secure --checkers=securityKeys,dbPrefix,adminUsername,versionInfo  --format=json
     *
     * @subcommand secure
     */
    public function secure( $args, $assoc_args ) {

        $formatter = new \WP_CLI\Formatter( $assoc_args );
        $checkersNames = isset($assoc_args['checkers']) ? $assoc_args['checkers'] : null;
        if (is_null($checkersNames)) {
            $formatter->display_item(array());
            return;
        }
        $checkers = explode(",", $checkersNames);

        foreach($checkers as $checker) {
            switch ($checker) {
                case 'dbPrefix':
                    ob_start();
                    WP_CLI::run_command( array( 'db-prefix', 'set' ), array('skip-success' => 'true'));
                    ob_get_clean();
                    $result[$checker] = array();
                    break;
                case 'securityKeys':
                    ob_start();
                    WP_CLI::run_command( array( 'security-keys', 'fix_keys' ), array('skip-success' => 'true'));
                    ob_get_clean();
                    $result[$checker] = array();
                    break;
                case 'versionInfo':
                    ob_start();
                    WP_CLI::run_command( array( 'version-info', 'secure' ), array('skip-success' => 'true'));
                    ob_get_clean();
                    $result[$checker] = array();
                    break;
                case 'adminUsername':
                    ob_start();
                    WP_CLI::run_command( array( 'admin-username', 'secure' ), array('format' => 'json', 'skip-error' => 'true') );
                    $resultString = ob_get_clean();
                    $result[$checker] = json_decode($resultString);
                    break;
                default:
                    break;
            }
        }

        $formatter->display_item( $result );
    }


}

WP_CLI::add_command( 'init', 'Init_Command' );